@@ -15,13 +15,13 @@ import httpx
 
 
 @contextmanager
-def sqlite_s3_query(url, get_credentials=lambda: (
+def sqlite_s3_query(url, get_credentials=lambda now: (
     os.environ['AWS_REGION'],
     os.environ['AWS_ACCESS_KEY_ID'],
     os.environ['AWS_SECRET_ACCESS_KEY'],
     os.environ.get('AWS_SESSION_TOKEN'),  # Only needed for temporary credentials
 ), get_http_client=lambda: httpx.Client(),
-   get_libsqlite3=lambda: cdll.LoadLibrary({'linux': 'libsqlite3.so', 'darwin': 'libsqlite3.dylib'}[platform])):
+   get_libsqlite3=lambda: cdll.LoadLibrary({'linux': 'libsqlite3.so.0', 'darwin': 'libsqlite3.dylib'}[platform])):
     libsqlite3 = get_libsqlite3()
     libsqlite3.sqlite3_errstr.restype = c_char_p
     libsqlite3.sqlite3_errmsg.restype = c_char_p
@@ -77,13 +77,14 @@ def sqlite_s3_query(url, get_credentials=lambda: (
 
     @contextmanager
     def make_auth_request(http_client, method, params, headers):
-        region, access_key_id, secret_access_key, session_token = get_credentials()
+        now = datetime.utcnow()
+        region, access_key_id, secret_access_key, session_token = get_credentials(now)
         to_auth_headers = headers + (
             (('x-amz-security-token', session_token),) if session_token is not None else \
             ()
         )
         request_headers = aws_sigv4_headers(
-            access_key_id, secret_access_key, region, method, to_auth_headers, params,
+            now, access_key_id, secret_access_key, region, method, to_auth_headers, params,
         )
         url = f'{scheme}://{netloc}{path}?{urlencode(params)}'
         with http_client.stream(method, url, headers=request_headers) as response:
@@ -91,60 +92,49 @@ def sqlite_s3_query(url, get_credentials=lambda: (
             yield response
 
     def aws_sigv4_headers(
-        access_key_id, secret_access_key, region, method, to_auth_headers, params,
+        now, access_key_id, secret_access_key, region, method, headers_to_sign, params,
     ):
+        def sign(key, msg):
+            return hmac.new(key, msg.encode('ascii'), sha256).digest()
+
         algorithm = 'AWS4-HMAC-SHA256'
 
-        now = datetime.utcnow()
         amzdate = now.strftime('%Y%m%dT%H%M%SZ')
-        datestamp = now.strftime('%Y%m%d')
+        datestamp = amzdate[:8]
         credential_scope = f'{datestamp}/{region}/s3/aws4_request'
 
-        to_auth_headers_lower = tuple((
-            (header_key.lower(), ' '.join(header_value.split()))
-            for header_key, header_value in to_auth_headers
-        ))
-        required_headers = (
+        headers = tuple(sorted(headers_to_sign + (
             ('host', netloc),
             ('x-amz-content-sha256', body_hash),
             ('x-amz-date', amzdate),
-        )
-        headers = sorted(to_auth_headers_lower + required_headers)
+        )))
         signed_headers = ';'.join(key for key, _ in headers)
 
-        def signature():
-            def canonical_request():
-                canonical_uri = quote(path, safe='/~')
-                quoted_params = sorted(
-                    (quote(key, safe='~'), quote(value, safe='~'))
-                    for key, value in params
-                )
-                canonical_querystring = '&'.join(f'{key}={value}' for key, value in quoted_params)
-                canonical_headers = ''.join(f'{key}:{value}\n' for key, value in headers)
+        canonical_uri = quote(path, safe='/~')
+        quoted_params = sorted(
+            (quote(key, safe='~'), quote(value, safe='~'))
+            for key, value in params
+        )
+        canonical_querystring = '&'.join(f'{key}={value}' for key, value in quoted_params)
+        canonical_headers = ''.join(f'{key}:{value}\n' for key, value in headers)
+        canonical_request = f'{method}\n{canonical_uri}\n{canonical_querystring}\n' + \
+                            f'{canonical_headers}\n{signed_headers}\n{body_hash}'
 
-                return f'{method}\n{canonical_uri}\n{canonical_querystring}\n' + \
-                       f'{canonical_headers}\n{signed_headers}\n{body_hash}'
+        string_to_sign = f'{algorithm}\n{amzdate}\n{credential_scope}\n' + \
+                         sha256(canonical_request.encode('ascii')).hexdigest()
 
-            def sign(key, msg):
-                return hmac.new(key, msg.encode('ascii'), sha256).digest()
-
-            string_to_sign = f'{algorithm}\n{amzdate}\n{credential_scope}\n' + \
-                             sha256(canonical_request().encode('ascii')).hexdigest()
-
-            date_key = sign(('AWS4' + secret_access_key).encode('ascii'), datestamp)
-            region_key = sign(date_key, region)
-            service_key = sign(region_key, 's3')
-            request_key = sign(service_key, 'aws4_request')
-            return sign(request_key, string_to_sign).hex()
+        date_key = sign(('AWS4' + secret_access_key).encode('ascii'), datestamp)
+        region_key = sign(date_key, region)
+        service_key = sign(region_key, 's3')
+        request_key = sign(service_key, 'aws4_request')
+        signature = sign(request_key, string_to_sign).hex()
 
         return (
-            (b'authorization', (
+            ('authorization', (
                 f'{algorithm} Credential={access_key_id}/{credential_scope}, '
-                f'SignedHeaders={signed_headers}, Signature=' + signature()).encode('ascii')
-             ),
-            (b'x-amz-date', amzdate.encode('ascii')),
-            (b'x-amz-content-sha256', body_hash.encode('ascii')),
-        ) + to_auth_headers
+                f'SignedHeaders={signed_headers}, Signature={signature}')
+            ),
+        ) + headers
 
     @contextmanager
     def get_vfs(http_client):
